"""Package for mine."""

__project__ = 'mine'
__version__ = '1.3'

CLI = 'mine'
VERSION = '{0} v{1}'.format(__project__, __version__)
DESCRIPTION = 'Share application state across computers using Dropbox.'
